@@ -1,31 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Quick 3D.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef QSSGCUSTOMMATERIAL_H
 #define QSSGCUSTOMMATERIAL_H
@@ -103,8 +77,6 @@ public:
     bool alwaysDirty() const;
     float lineWidth() const;
 
-    QVector<QQuick3DTexture *> dynamicTextureMaps() const;
-
 public Q_SLOTS:
     void setShadingMode(QQuick3DCustomMaterial::ShadingMode mode);
     void setVertexShader(const QUrl &url);
@@ -143,9 +115,11 @@ private Q_SLOTS:
 
 private:
     friend class QQuick3DShaderUtilsTextureInput;
-    void setDynamicTextureMap(QQuick3DTexture *textureMap, const QByteArray &name);
+    friend class QQuick3DViewport;
 
-    QVector<QQuick3DTexture *> m_dynamicTextureMaps;
+    void setDynamicTextureMap(QQuick3DShaderUtilsTextureInput *textureMap);
+
+    QSet<QQuick3DShaderUtilsTextureInput *> m_dynamicTextureMaps;
     quint32 m_dirtyAttributes = Dirty::AllDirty;
     BlendMode m_srcBlend = BlendMode::NoBlend;
     BlendMode m_dstBlend = BlendMode::NoBlend;
