@@ -1,32 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2008-2012 NVIDIA Corporation.
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Quick 3D.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2008-2012 NVIDIA Corporation.
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef QSSG_RHI_CUSTOM_MATERIAL_SYSTEM_H
 #define QSSG_RHI_CUSTOM_MATERIAL_SYSTEM_H
@@ -51,14 +25,13 @@
 #include <QtQuick3DRuntimeRender/private/qssgshadermapkey_p.h>
 #include <QtCore/qhash.h>
 #include <QtGui/QMatrix4x4>
-#include <QtGui/QMatrix3x3>
 
 QT_BEGIN_NAMESPACE
 
 struct QSSGRenderCustomMaterial;
 struct QSSGRenderSubset;
 struct QSSGRenderModel;
-struct QSSGLayerRenderData;
+class QSSGLayerRenderData;
 struct QSSGRenderableImage;
 struct QSSGRenderLayer;
 struct QSSGRenderLight;
@@ -70,12 +43,8 @@ class QRhiTexture;
 class Q_QUICK3DRUNTIMERENDER_EXPORT QSSGCustomMaterialSystem
 {
     Q_DISABLE_COPY(QSSGCustomMaterialSystem)
-public:
-    QAtomicInt ref;
-
-private:
     typedef QPair<QByteArray, QByteArray> TStrStrPair;
-    typedef QHash<QSSGShaderMapKey, QSSGRef<QSSGRhiShaderPipeline>> TShaderMap;
+    typedef QHash<QSSGShaderMapKey, QSSGRhiShaderPipelinePtr> TShaderMap;
 
     QSSGRenderContextInterface *context = nullptr;
     TShaderMap shaderMap;
@@ -85,7 +54,7 @@ private:
                             const QByteArray &inPropertyName,
                             const QVariant &propertyValue,
                             QSSGRenderShaderDataType inPropertyType,
-                            const QSSGRef<QSSGRhiShaderPipeline> &shaderPipeline);
+                            QSSGRhiShaderPipeline &shaderPipeline);
 
 public:
     QSSGCustomMaterialSystem();
@@ -93,33 +62,34 @@ public:
 
     void setRenderContextInterface(QSSGRenderContextInterface *inContext);
 
+    void releaseCachedResources();
+
     // Returns true if the material is dirty and thus will produce a different render result
     // than previously.  This effects things like progressive AA.
     bool prepareForRender(const QSSGRenderModel &inModel,
                           const QSSGRenderSubset &inSubset,
                           QSSGRenderCustomMaterial &inMaterial);
 
-    QSSGRef<QSSGRhiShaderPipeline> shadersForCustomMaterial(QSSGRhiGraphicsPipelineState *ps,
-                                                            const QSSGRenderCustomMaterial &material,
-                                                            QSSGSubsetRenderable &renderable,
-                                                            const QSSGShaderFeatures &featureSet);
+    QSSGRhiShaderPipelinePtr shadersForCustomMaterial(QSSGRhiGraphicsPipelineState *ps,
+                                                      const QSSGRenderCustomMaterial &material,
+                                                      QSSGSubsetRenderable &renderable,
+                                                      const QSSGShaderFeatures &featureSet);
 
-    void updateUniformsForCustomMaterial(QSSGRef<QSSGRhiShaderPipeline> &shaderPipeline,
+    void updateUniformsForCustomMaterial(QSSGRhiShaderPipeline &shaderPipeline,
                                          QSSGRhiContext *rhiCtx,
                                          char *ubufData,
                                          QSSGRhiGraphicsPipelineState *ps,
                                          const QSSGRenderCustomMaterial &material,
                                          QSSGSubsetRenderable &renderable,
-                                         QSSGLayerRenderData &layerData,
-                                         QSSGRenderCamera &camera,
+                                         const QSSGRenderCamera &camera,
                                          const QVector2D *depthAdjust,
                                          const QMatrix4x4 *alteredModelViewProjection);
 
-    void rhiPrepareRenderable(QSSGRhiGraphicsPipelineState *ps,
+    void rhiPrepareRenderable(QSSGRhiGraphicsPipelineState *ps, QSSGPassKey passKey,
                               QSSGSubsetRenderable &renderable,
                               const QSSGShaderFeatures &featureSet,
                               const QSSGRenderCustomMaterial &material,
-                              QSSGLayerRenderData &layerData,
+                              const QSSGLayerRenderData &layerData,
                               QRhiRenderPassDescriptor *renderPassDescriptor,
                               int samples,
                               QSSGRenderCamera *camera = nullptr,
@@ -128,13 +98,12 @@ public:
                               QSSGReflectionMapEntry *entry = nullptr);
     void applyRhiShaderPropertyValues(char *ubufData,
                                       const QSSGRenderCustomMaterial &inMaterial,
-                                      const QSSGRef<QSSGRhiShaderPipeline> &shaderPipeline);
+                                      QSSGRhiShaderPipeline &shaderPipeline);
     void rhiRenderRenderable(QSSGRhiContext *rhiCtx,
                              QSSGSubsetRenderable &renderable,
-                             QSSGLayerRenderData &inData,
                              bool *needsSetViewport,
                              int cubeFace,
-                             QSSGRhiGraphicsPipelineState *state);
+                             const QSSGRhiGraphicsPipelineState &state);
 };
 
 QT_END_NAMESPACE
